# lab2_group_analysis.R
# PSYCH 4165 Lab 2
# Command file for using lme() to compute a
# repeated measures analysis of variance in R.
# Lew Harvey
# 11 February 2014

# if you want to use lmer() instead of lme(), set this flag to TRUE
USE_LME4 = FALSE
if (USE_LME4) {
	if(!require(lme4))	# load the lme4 package to use lmer()
	{
		print("Download package 'lme4' from CRAN to use lmer()")
	}
} else {
	if(!require(nlme))	# load the nlme package to use lme()
	{
		print("Download package 'nlme' from CRAN to use lme()")
	}
}

# if you want to do effect size analysis, set this flag to TRUE
COMPUTE_EFFECT_SIZE = TRUE
if (COMPUTE_EFFECT_SIZE) {
	if(!require(bootES)) {
		print("Download package 'bootES' from CRAN to use bootES()")
	}
}

# Read in group data in wide format from text file:
# fn <- file("lab2_group_data_wide_2014_Spring.csv")
fn <- file.choose()
df.wide <- read.csv(fn, header=TRUE)

# add within subject eyes effect size on d-sub-a to data frame
df.wide$eyes.effect <- (df.wide$da_with_eyes - df.wide$da_no_eyes) / df.wide$da_with_eyes
# add within subject model effect size on aic to data frame
df.wide$model.effect <- df.wide$htm_AIC - df.wide$sdt_AIC

# ******************************************************
# reshape the file into long format
# Note: reshape() is a bit of a pain to use but what it does
# is take data that are in a so-called wide format
# (one row per subject, separate columns per dependent variable)
# and put it into the long format needed by many R analysis routines
# (multiple rows per subject, single column per dependent variable)
df.long <- reshape(df.wide,
	varying = list(c("da_no_eyes", "da_with_eyes"), c("htm_no_eyes", "htm_with_eyes"), c("sdt_AIC", "htm_AIC")),
	v.names = c("da", "htm_p", "aic"),
	timevar = c("eyes", "mod"),
	idvar = "subj",
	times = c("noEyes", "withEyes"),
	direction = "long")
	
rownames(df.long) <- 1:dim(df.long)[1]
df.long[df.long$mod == "noEyes", "mod"] <- "sdt"
df.long[df.long$mod == "withEyes", "mod"] <- "htm"
df.long$eyes <- factor(df.long$eyes)
df.long$mod <- factor(df.long$mod)
df.long$section <- factor(df.long$section)
df <- df.long

# for drawing lines between pairs of data
df.sdt		<- subset(df, mod=="sdt")
df.htm		<- subset(df, mod=="htm")
df.withEyes <- subset(df, eyes=="withEyes")
df.noEyes	<- subset(df, eyes=="noEyes")

# ******************************************************
# make functions to draw strip and box plots
# plot1()  plots AIC fit against the two models
# plot2()  plots memory sensitivity for eyes/no eyes
# ******************************************************

# Write out a summary of the variables in the data frame:
print(summary(df))

# ******************************************************
# plotting section
# define four functions to plot the two sets of graphs
# plot1()
# plot1a()
# plot2()
# plot2a()
# To make the plots, type the command
# ******************************************************

plot1 <- function() {
	# set graphic parameters to plot two graphs in one panel
	par(mfcol=c(1,2))	
	# Make a strip chart of the data
	stripchart(aic ~  mod, data = df,
		vertical = TRUE,
		method = "jitter",  jitter = 0.03,
		xlim = c(0.75, 2.25),
		xlab = "Detection Model",
		ylab = "Badness-of-Fit (AIC)",
		main = "Lab 2 Group Data")
	
	# Make a box plot of the data:
	boxplot(aic ~ mod, data = df,
		col = "yellow",
		horizontal = FALSE,
		xlab = "Detection Model",
		ylab = "Badness-of-Fit (AIC)",
		main = "Lab 2 Group Data")

	par(mfcol=c(1,1))
}	

# plot1a draws the same plot as plot1() but adds
# lines connecting data points from the same subject
plot1a <- function() {
	par(mfcol=c(1,2))	# two plots per figure
	# Make a strip chart of the data
	stripchart(aic ~  mod, data = df,
		vertical = TRUE,
		method = "jitter",  jitter = 0.03,
		xlim = c(0.75, 2.25),
		xlab = "Detection Model",
		ylab = "Badness-of-Fit (AIC)",
		main = "Lab 2 Group Data")
		
	# connect corresponding points with a line
	segments(x0 = rep(1, length(df.htm$aic)), y0 = df.htm$aic, 
			 x1 = rep(2, length(df.sdt$aic)), y1 = df.sdt$aic)
	
	# Make a box plot of the data:
	boxplot(aic ~ mod, data = df,
		col = "yellow",
		horizontal = FALSE,
		xlab = "Detection Model",
		ylab = "Badness-of-Fit (AIC)",
		main = "Lab 2 Group Data")

	par(mfcol=c(1,1))
}


# Test the hypothesis that eyes have no influence on face memory
# Memory sensitivity is measured with d-sub-a, the generalized
# signal detection theory measure d-prime
plot2 <- function() {
	par(mfcol=c(1,2))	# two plots per figure
	# Make a strip chart of the data
	stripchart(da ~ eyes, data = df, vertical = TRUE,
		method = "jitter", jitter = 0.03,
		xlim = c(0.75, 2.25),
		xlab = "Eyes (Yes or No)",
		ylab = "Face Memory Sensitivity (Da)",
		main = "Lab 2 Group Data")
	
	# Make a box plot of the data
	boxplot(da ~ eyes, data = df, horizontal = FALSE,
		col = "yellow",
		xlab = "Eyes (Yes or No)",
		ylab = "Face Memory Sensitivity (Da)",
		main = "Lab 2 Group Data")

	par(mfcol=c(1,1))	# two plots per figure
}	

plot2a <- function() {
	par(mfcol=c(1,2))	# two plots per figure
	# Make a strip chart of the data
	stripchart(da ~ eyes, data = df, vertical = TRUE,
		method = "jitter", jitter = 0.03,
		xlim = c(0.75, 2.25),
		xlab = "Eyes (Yes or No)",
		ylab = "Face Memory Sensitivity (Da)",
		main = "Lab 2 Group Data")
		
	# connect corresponding points with a line
	segments(x0 = rep(1, length(df.noEyes$da)),   y0 = df.noEyes$da,
			 x1 = rep(2, length(df.withEyes$da)), y1 = df.withEyes$da)

	# Make a box plot of the data
	boxplot(da ~ eyes, data = df, horizontal = FALSE,
		col = "yellow",
		xlab = "Eyes (Yes or No)",
		ylab = "Face Memory Sensitivity (Da)",
		main = "Lab 2 Group Data")

	par(mfcol=c(1,1))	# one plot per figure
}


# plot the effect size of the difference between da with eyes and da without eyes
plot3 <- function() {
	par(mfcol=c(1,2))	# two plots per figure
	# Make a strip chart of the data
	stripchart(df.wide$eyes.effect, vertical = TRUE,
		method = "jitter", jitter = 0.03,
		xlab = "Effect of Eyes on Recognition",
		ylab = "Eye Effect Index",
		main = "Lab 2 Group Data")
	abline(h = 0, col = "red")
	
	# Make a box plot of the data
	boxplot(df.wide$eyes.effect, horizontal = FALSE,
		col = "yellow",
		xlab = "Effect of Eyes on Recognition",
		ylab = "Eye Effect Index",
		main = "Lab 2 Group Data")
	abline(h = 0, col = "red")

	par(mfcol=c(1,1))	# two plots per figure
}	

# test the hypothesis that viewing the eyes does not influence memory
# Compute the repeated measures ANOVA to test the null hypothesis
# that having eyes or no-eyes does not influence memory sensitivity (d-sub-a)

if (USE_LME4) {
	print("", quote=FALSE)
	print("d-sub-a linear mixed effects model (lmer)")
	lmer.mod.da <- lmer(da ~ eyes + (1 | subj), data = df)
	print(anova(lmer.mod.da))
} else {
	print("", quote=FALSE)
	print("d-sub-a linear mixed effects model (lme)")
	lme.mod.da <- lme(da ~ eyes, random = ~ 1 | subj, data = df)
	print(anova(lme.mod.da))
}

# print group means
print("", quote=FALSE)
print("d-sub-a Means")
with(df, print(cbind(mean = tapply(da, eyes, mean),
	std.dev = tapply(da, eyes, sd),
	n = tapply(da, eyes, length))))



# test the hypothesis that the two models
# fit the data equally well using AIC as the
# badness of fit measure: the bigger the AIC,
# the worse the fit of the model.	
# Compute a repeated measures ANOVA

if (USE_LME4) {
	print("", quote = FALSE)
	print("Badness-of-Fit linear mixed effects model (lmer)")
	lmer.mod.aic <- lmer(aic ~ mod + (1 | subj), data = df)
	print(anova(lmer.mod.aic))
} else {
	print("", quote = FALSE)
	print("Badness-of-Fit linear mixed effects model (lme)")
	lme.mod.aic <- lme(aic ~ mod, random = ~ 1 | subj, data = df)
	print(anova(lme.mod.aic))
}

# print group means
with(df, print(cbind(mean = tapply(aic, mod, mean),
	std.dev = tapply(aic, mod, sd),
	n = tapply(aic, mod, length))))


plot2()

# effect size and confidence interval
if (COMPUTE_EFFECT_SIZE) {
	# eyes effect on da
	da.bes.unstandardized	<- bootES(data = df.wide, R = 10000, data.col = "eyes.effect", effect.type = "unstandardized", plot = FALSE)
	da.bes.cohens.d			<- bootES(data = df.wide, R = 10000, data.col = "eyes.effect", effect.type = "cohens.d", plot = FALSE)
	da.bes.cohens.d.sigma	<- bootES(data = df.wide, R = 10000, data.col = "eyes.effect", effect.type = "cohens.d.sigma", plot = FALSE)
	da.bes.hedges.g			<- bootES(data = df.wide, R = 10000, data.col = "eyes.effect", effect.type = "hedges.g", plot = FALSE)
	
	# model effect on AIC
	aic.bes.unstandardized	<- bootES(data = df.wide, R = 10000, data.col = "model.effect", effect.type = "unstandardized", plot = FALSE)
	aic.bes.cohens.d		<- bootES(data = df.wide, R = 10000, data.col = "model.effect", effect.type = "cohens.d", plot = FALSE)
	aic.bes.cohens.d.sigma	<- bootES(data = df.wide, R = 10000, data.col = "model.effect", effect.type = "cohens.d.sigma", plot = FALSE)
	aic.bes.hedges.g		<- bootES(data = df.wide, R = 10000, data.col = "model.effect", effect.type = "hedges.g", plot = FALSE)
}